/**
   @file collection.h 
   @brief A general purpose data structure
*/
/*
   Copyright (C)2001, 2002 Anthony Arcieri 
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

 * Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.

 * The name of Anthony Arcieri may not be used to endorse or promote 
 products derived from this software without specific prior written 
 permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT 
 OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef COLLECTION_H
#define COLLECTION_H

#include "machdep.h"

/** 
 * @defgroup collection Collection
 * @{
 */
/* Data type abstraction */
/** key */
typedef uint32_t pdtp_collection_key_t;
/** the collection handle */
typedef struct pdtp_collection *pdtp_collection_t;

/** collection_create:
   @return a handle to a new collection 
*/
pdtp_collection_t pdtp__collection_create(void);

/** 
 * @brief Destroys the collection.  
 * @remark Calls the given free function on every pointer stored in the collection.
 */ 
void pdtp__collection_destroy(pdtp_collection_t, void (*free_func)(void *));

/** 
@brief Inserts a key and an associated pointer into the given collection.

@retval 0 Entry successfully inserted
@retval -1 Entry already exists
 */ 
int pdtp__collection_insert(pdtp_collection_t, pdtp_collection_key_t, void *);

/** 
   @brief Removes the entry associated with the given key from the collection 
   @retval ptr  the associated pointer assocatied with the given key. 
   @retval NULL If no entry is found in the collection.
*/ 
void *pdtp__collection_remove(pdtp_collection_t, pdtp_collection_key_t);

/** collection_lookup:
   @return the pointer associated with the given key, or NULL if not found.
 */
void *pdtp__collection_lookup(pdtp_collection_t, pdtp_collection_key_t);

/** collection_iterate description:
   Performs an in-order traversal of the collection, calling the given function
   with the data associated with each key as an argument.  
   The function should return 1 if continued iteration is desired, or 0 to stop iteration early.  
   A pointer to pass to the callback function may also be given.
 */
void pdtp__collection_iterate(pdtp_collection_t, 
		int (*iterator)(pdtp_collection_key_t, void *data, void *ptr), 
		void *ptr);

/** collection_size:
   @return the number of items in the collection.
*/
int pdtp__collection_size(pdtp_collection_t);

/** @} */
#endif /* COLLECTION_H */
