#ifndef HASHER_H
#define HASHER_H
/**
 * @file hasher.h 
 * @brief Abstract hash cipher interface
 */

/** available algorithm types for the hash */
typedef enum {
	PDTP_HASH_MD5, /**< MD5 */
	PDTP_HASH_SHA1 /**< SHA1*/
} pdtp_hash_t;

/** handle to the hasher */
typedef struct pdtp_hasher *pdtp_hasher_t;

/** 
 * @brief create a hasher 
 * @param algorithm he algorithm to use
 * @returns the hasher
 */
pdtp_hasher_t pdtp__hasher_create(pdtp_hash_t algorithm);
/**
 * @brief destroy the hasher 
 */
void pdtp__hasher_destroy(pdtp_hasher_t);

/** 
 * @brief return the digest size
 * @return the size
 */
unsigned pdtp__hasher_digest_size(pdtp_hasher_t);

/**
 * @brief initalize the hasher
 */
void pdtp__hasher_zero(pdtp_hasher_t);

/**
 * @brief add data to the hasher
 * @param buf the data to add
 * @param len size of the buffer
 */
void pdtp__hasher_add_data(pdtp_hasher_t, void *buf, unsigned len);

/**
 * @brief finish computing the hasher and return the digest
 * @param digest the digest returned
 */
 
void pdtp__hasher_get_digest(pdtp_hasher_t, void *digest);

#endif
