/*
   hmac.c - RFC2104 keyed hashing for message authentication
   Copyright (C)2004 Anthony Arcieri 
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

 * Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.

 * The name of Anthony Arcieri may not be used to endorse or promote 
 products derived from this software without specific prior written 
 permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT 
 OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "machdep.h"

#include <string.h>

#include "alloc.h"
#include "hasher.h"
#include "hmac.h"

/* Valid for MD5 and SHA1 according to RFC2104 */
#define BLOCK_SIZE	64

void pdtp__hmac_compute(pdtp_hash_t algorithm, void *key, unsigned klen, void *data, unsigned dlen, void *digest)
{
	int i;
	void *kptr;
	uint8_t ipad[BLOCK_SIZE], opad[BLOCK_SIZE];
	pdtp_hasher_t hasher = pdtp__hasher_create(algorithm);

	if(klen > BLOCK_SIZE) {
		pdtp__hasher_zero(hasher);
		pdtp__hasher_add_data(hasher, key, klen);

		klen = pdtp__hasher_digest_size(hasher);
		kptr = (uint8_t *)pdtp__alloc(klen);
		
		pdtp__hasher_get_digest(hasher, kptr);
	} else
		kptr = key;
		
	memset(ipad, 0, BLOCK_SIZE);
	memset(opad, 0, BLOCK_SIZE);
	memcpy(ipad, kptr, klen);
	memcpy(opad, kptr, klen);

	if(key != kptr)
		pdtp__free(kptr);

	for(i = 0; i < BLOCK_SIZE; i++) {
		ipad[i] ^= 0x36;
		opad[i] ^= 0x5c;
	}

	pdtp__hasher_zero(hasher);
	pdtp__hasher_add_data(hasher, ipad, BLOCK_SIZE);
	pdtp__hasher_add_data(hasher, data, dlen);
	pdtp__hasher_get_digest(hasher, digest);

	pdtp__hasher_zero(hasher);
	pdtp__hasher_add_data(hasher, opad, BLOCK_SIZE);
	pdtp__hasher_add_data(hasher, digest, pdtp__hasher_digest_size(hasher));
	pdtp__hasher_get_digest(hasher, digest);

	pdtp__hasher_destroy(hasher);
}
