#!/bin/sh

# Use GCC on platforms that ship with it standard
case `uname` in
Darwin)
	CC=gcc
	CFLAGS="-g -Wall -fPIC" # MacOS X's headers aren't -ansi -pedantic clean
;;
Linux)
	CC=gcc
	CFLAGS="-ansi -pedantic -fpic -g -Wall -D_GNU_SOURCE"
;;
FreeBSD|OpenBSD|NetBSD)
	CC=gcc
	CFLAGS="-ansi -pedantic -fpic -g -Wall"
;;
SunOS)
	# Try 'cc' first, then fall back on 'gcc' 
	if [ -x `which cc` ]; then
		CC=cc
	else
		CC=gcc
	fi

	CFLAGS="-D_REENTRANT -D_POSIX_PTHREAD_SEMANTICS"
;;
*)
	# Try 'cc' first, then fall back on 'gcc'
	if [ -x `which cc` ]; then 
		CC=cc
	else
		CC=gcc
	fi

	CFLAGS=
;;	
esac	

if [ ! -x `which $CC` ]; then
	printf "error: Couldn't find '$CC'"
	exit 1
fi

# Append extra compile arguments generated by ./configure
CFLAGS="$CFLAGS $@"

printf "using: $CC $CFLAGS\n"

( printf "#!/bin/sh\n"; \
  printf "exec $CC $CFLAGS \${1+\"\$@\"}\n" \
) > compile

chmod +x compile
exit 0

printf "error: Couldn't find '$CC'"
exit 1
