#!/bin/sh

OS=`uname`
REL=`uname -r`

if [ "$OS" = "Linux" ]; then
	ARCH=`uname -m`
else	
	ARCH=`uname -p`
fi	

echo "#ifndef MACHDEP_H" > machdep.h
echo "#define MACHDEP_H" >> machdep.h
echo "/** @file machdep.h" >> machdep.h
echo "* @brief machine dependancys (auto-generated)">> machdep.h
echo "*/" >> machdep.h
echo >> machdep.h

case "$OS" in
Darwin)
	echo "configuring for MacOS X ($OS $REL) on $ARCH"
	echo "#define HAVE_GETADDRINFO" >> machdep.h
		echo "/** system has pthread_* functions */" >> machdep.h
	echo "#define HAVE_PTHREADS" >> machdep.h
	echo "#define HAVE_UIO" >> machdep.h
	echo "#define HAVE_VASPRINTF" >> machdep.h
	if [ -f /usr/include/sys/poll.h ]; then
		echo "/** system has poll() function */" >> machdep.h
		echo "#define HAVE_POLL" >> machdep.h
	fi
	;;
SunOS)
	echo "configuring for Solaris ($OS $REL) on $ARCH"
	echo "#define HAVE_GETADDRINFO" >> machdep.h
    echo "/** system has pthread_* functions */" >> machdep.h
	echo "#define HAVE_PTHREADS" >> machdep.h
	echo "#define HAVE_UIO" >> machdep.h
    echo "/** system has poll() function */" >> machdep.h
	echo "#define HAVE_POLL" >> machdep.h
	echo "#define HAVE_SOCKLEN_T" >> machdep.h
	;;
Linux)
	echo "configuring for $OS $REL on $ARCH"
	echo "#define HAVE_GETADDRINFO" >> machdep.h
    echo "/** system has pthread_* functions */" >> machdep.h
	echo "#define HAVE_PTHREADS" >> machdep.h
	echo "#define HAVE_UIO" >> machdep.h
	echo "#define HAVE_VASPRINTF" >> machdep.h
    echo "/** system has poll() function */" >> machdep.h
	echo "#define HAVE_POLL" >> machdep.h
	echo "#define HAVE_SOCKLEN_T" >> machdep.h
	;;
FreeBSD|OpenBSD|NetBSD)
	echo "configuring for $OS $REL on $ARCH"
	echo "#define HAVE_GETADDRINFO" >> machdep.h
    echo "/** system has pthread_* functions */" >> machdep.h
	echo "#define HAVE_PTHREADS" >> machdep.h
	echo "#define HAVE_UIO" >> machdep.h
	echo "#define HAVE_VASPRINTF" >> machdep.h
    echo "/** system has poll() function */" >> machdep.h
	echo "#define HAVE_POLL" >> machdep.h
	echo "#define HAVE_SOCKLEN_T" >> machdep.h
	;;
*)
	echo "unsupported OS: $OS $REL"
	echo "*** You may need to hand edit machdep.h"
	;;
esac

case "$ARCH" in
powerpc|sparc|SUN*|mips)
        echo "Configuring for a big endian processor architecture: $ARCH"
        echo "/** big endian processor architecture: $ARCH */" >>machdep.h
        echo "#define HOST_BIGENDIAN 1" >> machdep.h
        ;;
*)
        echo "Configuring for a little endian processor architecture: $ARCH"
        echo "/** little endian processor architecture: $ARCH */" >>machdep.h
        echo "/* #undef HOST_BIGENDIAN */" >> machdep.h
        ;;
esac

echo >> machdep.h
echo "#include <sys/types.h>" >> machdep.h
echo "#include <inttypes.h>" >> machdep.h
echo >> machdep.h
echo "#endif" >> machdep.h

exit 0
