#ifndef MULTIPLEXER_H
#define MULTIPLEXER_H

#include "notifier.h"
/** 
 * @file multiplexer.h 
 * @brief Platform independent socket multiplexer mechanisms
 */

typedef struct pdtp_multiplexer *pdtp_multiplexer_t;
typedef enum { MPLX_READ, MPLX_WRITE } pdtp_event_t;

/** create a multiplexer object.  poll() will be used
   on POSIX systems if it is available, otherwise a fallback select()-based
   implementation is provided.  On Windows platforms, WSAEventSelect() is
   used, but unfortunately this implementation only scales up to 64
   sockets due to limitations of WSAWaitForMultipleObjects. */
pdtp_multiplexer_t pdtp__multiplexer_create(void);
/** destroy the multiplexer object */
void pdtp__multiplexer_destroy(pdtp_multiplexer_t);

/** This function adds the specified socket to the multiplexer after the given
   millisecond delay.  The delay allows intelligent bandwidth throttling
   to be applied, as readability/writability will not be checked until
   the delay has elapsed.  If a read/write event occurs, the given
   function callback will be executed with the given argument */
void pdtp__multiplexer_add_socket(pdtp_multiplexer_t, int sock, pdtp_event_t, int (*function)(pdtp_multiplexer_t, int sock, void *arg), void *arg, int msec_delay);

/** This removes a socket from the multiplexer.  The arg paramater originally
   passed to pdtp__multiplexer_add_socket() is returned */
void *pdtp__multiplexer_remove_socket(pdtp_multiplexer_t, int sock);

/** This socket works like the one above, except it adds a notifier instead
   of a socket */
void pdtp__multiplexer_add_notifier(pdtp_multiplexer_t, pdtp_notifier_t, int (*function)(pdtp_multiplexer_t, pdtp_notifier_t, void *arg), void *arg);

/** This removes a notifier from the multiplexer.  The arg paramater originally
   passed to pdtp__multiplexer_add_notifier() is returned */
void *pdtp__multiplexer_remove_notifier(pdtp_multiplexer_t, pdtp_notifier_t);

/**  run the multiplexer */
void pdtp__multiplexer_run(pdtp_multiplexer_t);

#endif
