#ifndef SOCKETOPS_H
#define SOCKETOPS_H

/**
 * @file socketops.h 
 * @brief Platform independent socket functions
 */
#include "machdep.h"
#include "sockets.h"

#include "multiplexer.h"

#ifdef WIN32
typedef SOCKET pdtp_sock_t;
#else
typedef int pdtp_sock_t;
#endif /* WIN32 */

typedef struct pdtp_addr *pdtp_addr_t;
typedef enum { PDTP_SOCKET_BLOCK, PDTP_SOCKET_NONBLOCK } pdtp_blockmode_t;

pdtp_addr_t pdtp__ipv4_addr_create(uint32_t addr, uint16_t port);
pdtp_addr_t pdtp__resolve(const char *hostname, int port);
void pdtp__addr_destroy(pdtp_addr_t);
uint32_t pdtp__addr_ipv4(pdtp_addr_t);
uint16_t pdtp__addr_port(pdtp_addr_t);

pdtp_sock_t pdtp__nonblock_connect(pdtp_addr_t addr);
int pdtp__nonblock_connect_complete(pdtp_sock_t sock);
pdtp_sock_t pdtp__connect(pdtp_addr_t);

int pdtp__set_blocking_mode(pdtp_sock_t sock, pdtp_blockmode_t mode);

ssize_t pdtp__atomic_readsock(pdtp_sock_t sock, void *buf, size_t nbytes);
ssize_t pdtp__timeout_readsock(pdtp_sock_t sock, void *buf, size_t nbytes);
int pdtp__readsock(pdtp_sock_t sock, void *buf, size_t nbytes);
int pdtp__readsock_int16(pdtp_sock_t sock, uint16_t *value);
int pdtp__readsock_int32(pdtp_sock_t sock, uint32_t *value);

ssize_t pdtp__atomic_writesock(pdtp_sock_t sock, void *buf, size_t nbytes);
ssize_t pdtp__timeout_writesock(pdtp_sock_t sock, void *buf, size_t nbytes);
int pdtp__writesock(pdtp_sock_t sock, void *buf, size_t nbytes);
int pdtp__writesock_int16(pdtp_sock_t sock, uint16_t value);
int pdtp__writesock_int32(pdtp_sock_t sock, uint32_t value);

void pdtp__nonblock_read(pdtp_multiplexer_t mplx, pdtp_sock_t sock, unsigned length, int (*completion_function)(pdtp_multiplexer_t mplx, int sock, void *buffer, int bytes, void *arg), void *arg);
void pdtp__nonblock_write(pdtp_multiplexer_t mplx, pdtp_sock_t sock, void *buffer, unsigned length, int (*completion_function)(pdtp_multiplexer_t mplx, int sock, void *buffer, int bytes, void *arg), void *arg);

#ifdef WIN32
#define pdtp__socket_close(sock)       closesocket(sock)
#else
#define pdtp__socket_close(sock)       close(sock)
#endif

pdtp_sock_t pdtp__ipv4_tcp_listener_create(uint32_t source_addr, uint16_t port);
pdtp_sock_t pdtp__ipv4_listener_accept(pdtp_sock_t sock, uint32_t *addr);
	
#endif /* SOCKETOPS_H */
