#ifndef THREADOPS_H
#define THREADOPS_H

/** 
 * @file threadops.h
 * @brief Win32/POSIX thread abstraction layer
 */
#include "machdep.h"

#ifdef HAVE_PTHREADS
#include <pthread.h>
#endif

#ifdef HAVE_PTHREADS
/** mutex lock */
typedef pthread_mutex_t *pdtp_mutex_t;
/** conditional lock */
typedef pthread_cond_t *pdtp_cond_t;
#endif
#ifdef WIN32
typedef LPCRITICAL_SECTION pdtp_mutex_t;
typedef HANDLE pdtp_cond_t;
#endif

/** create a mutex */
pdtp_mutex_t pdtp__mutex_create(void);
/** destroy the  mutex */
void pdtp__mutex_destroy(pdtp_mutex_t);

/** create a conditional lock */
pdtp_cond_t pdtp__cond_create(void);
/** destroy the conditional lock */
void pdtp__cond_destroy(pdtp_cond_t);

#ifdef WIN32
#define pdtp__mutex_lock(mutex)		EnterCriticalSection(mutex)
#define pdtp__mutex_unlock(mutex)	LeaveCriticalSection(mutex)

int pdtp__cond_wait(pdtp_cond_t, pdtp_mutex_t);
#define pdtp__cond_signal(cond)		PulseEvent(cond)
#endif /* WIN32 */

#ifdef HAVE_PTHREADS
/** acquire the mutex lock */
#define pdtp__mutex_lock(mutex)		pthread_mutex_lock(mutex)
/** relinquish the mutex lock */
#define pdtp__mutex_unlock(mutex)	pthread_mutex_unlock(mutex)

/** wait on a condition */
#define pdtp__cond_wait(cond, mutex)	pthread_cond_wait(cond, mutex)
/** signal the condition lock */
#define pdtp__cond_signal(cond)		pthread_cond_signal(cond)
#endif /* HAVE_PTHREADS */

#if !defined(WIN32) && !defined(HAVE_PTHREADS)
int pdtp__mutex_lock(pdtp_mutex_t mutex);
int pdtp__mutex_unlock(pdtp_mutex_t mutex);
int pdtp__cond_wait(pdtp_cond_t cond, pdtp_mutex_t mutex);
int pdtp__cond_signal(pdtp_cond_t cond);
#endif /* !defined(WIN32) && !defined(HAVE_PTHREADS) */

#endif /* THREADOPS_H */
